@@ -62,6 +62,9 @@ sub create_from_hash
 {
         my ($class, $hash, $specials_accepted) = @_;
 
+        if (exists($hash->{caltype}) && ($hash->{caltype} eq 'weekly')) {
+                return Remind::PDF::Weekly->create_from_hash($hash, $specials_accepted);
+        }
         bless $hash, $class;
 
         my $filtered_entries = [];
@@ -463,8 +466,16 @@ C<rem2pdf> for the contents of C<$settings>
 =cut
 sub render
 {
-        my ($self, $cr, $settings) = @_;
+        my ($self, $cr, $settings, $index, $total) = @_;
 
+        if ($settings->{svg} || $settings->{eps}) {
+                if ($index > 1) {
+                        if ($index == 2) {
+                                print STDERR "WARNING: --svg/--eps can only output one page; ignoring subsequent\nmonths in a multi-month calendar.\n";
+                        }
+                        return;
+                }
+        }
         $self->setup_daymap($settings);
         $self->{horiz_lines} = [];
         $cr->set_line_cap('square');
@@ -525,7 +536,7 @@ sub render
         }
 
         if ($settings->{verbose}) {
-                print STDERR "remdp2f: Rendered " . $self->{monthname} . ' ' . $self->{year} . "\n";
+                print STDERR "rem2pdf: Rendered " . $self->{monthname} . ' ' . $self->{year} . "\n";
         }
         # Done this page
         $cr->show_page();
@@ -1008,20 +1019,260 @@ as were read from the C<remind -ppp> stream
 sub render
 {
         my ($self, $cr, $settings) = @_;
-        my $done = 0;
-        my $warned = 0;
+        my $index = 0;
+        my $total = scalar(@{$self->{entries}});
+
         foreach my $e (@{$self->{entries}}) {
-                if ($settings->{svg} && $done) {
-                        if (!$warned) {
-                                print STDERR "WARNING: --svg can only output one page; ignoring subsequent\nmonths in a multi-month calendar.\n";
-                                $warned = 1;
+                $index++;
+                $e->render($cr, $settings, $index, $total);
+        }
+}
+
+package Remind::PDF::Weekly;
+use base qw(Remind::PDF);
+
+=head1 NAME
+
+Remind::PDF::Weekly - render a weekly calendar
+
+=cut
+
+sub render
+{
+        my ($self, $cr, $settings, $index, $total) = @_;
+        if ($settings->{svg} || $settings->{eps}) {
+                if ($index > $settings->{weeks_per_page}) {
+                        if ($index == $settings->{weeks_per_page}+1) {
+                                print STDERR "WARNING: --svg/--eps can only output one page; ignoring subsequent pages.\n";
                         }
+                        return;
+                }
+        }
+
+        $settings->{numbers_on_left} = 1;
+        # Set up bounding box
+        if ($settings->{weeks_per_page} == 1) {
+                $self->{bounding_box} = [
+                        $settings->{margin_left},
+                        $settings->{margin_top},
+                        $settings->{width} - $settings->{margin_right},
+                        $settings->{height} - $settings->{margin_bottom}]
+        } else {
+                my $total_height = $settings->{height} - $settings->{margin_top} - $settings->{margin_bottom};
+                my $week_height = $total_height / $settings->{weeks_per_page};
+                my $top_offset = (($index-1) % $settings->{weeks_per_page}) * $week_height;
+                my $bot_offset = $top_offset + $week_height;
+                $self->{bounding_box} = 
+                    $self->{bounding_box} = [
+                            $settings->{margin_left},
+                            $settings->{margin_top} + $top_offset,
+                            $settings->{width} - $settings->{margin_right},
+                            $settings->{margin_top} + $bot_offset];
+                if ($index != 1) {
+                        $self->{bounding_box}[1] += 0.25 * $settings->{margin_top};
+                }
+                if ($index != $settings->{weeks_per_page}) {
+                    $self->{bounding_box}[3] -= 0.25 * $settings->{margin_top};
+                }
+        }
+        $self->draw_headings($cr, $settings);
+        for (my $i=0; $i<7; $i++) {
+                $self->draw_entries($cr, $settings, $i);
+        }
+        $self->draw_lines($cr, $settings);
+        if ($index == $total || ($index % $settings->{weeks_per_page}) == 0) {
+                $cr->show_page();
+        }
+
+        if ($settings->{verbose}) {
+                print STDERR "rem2pdf: Rendered " . $self->{dates}->[0]->{date} . " to " . $self->{dates}->[6]->{date} . "\n";
+        }
+}
+
+sub draw_headings
+{
+        my ($self, $cr, $settings) = @_;
+        my $ymax = 0;
+        my $cell = ($settings->{width} - $settings->{margin_left} - $settings->{margin_right})/7;
+
+        for (my $i=0; $i<7; $i++) {
+                my $date = $self->{dates}[$i];
+                my $month = $date->{month};
+                my $year = $date->{year};
+                my $day = $date->{day};
+                my $dayname = $date->{dayname};
+
+                my $layout = Pango::Cairo::create_layout($cr);
+                $layout->set_text(Encode::decode('UTF-8', $dayname));
+
+                my $desc = Pango::FontDescription->from_string($settings->{header_font} . ' ' . $settings->{header_size} . 'px');
+                $layout->set_font_description($desc);
+
+                my ($wid, $h) = $layout->get_pixel_size();
+                $cr->save;
+                $cr->move_to($settings->{margin_left} + $i * $cell + $cell/2 - $wid/2, $self->{bounding_box}[1]);
+                Pango::Cairo::show_layout($cr, $layout);
+                $cr->restore();
+
+                $layout = Pango::Cairo::create_layout($cr);
+                $layout->set_text(Encode::decode('UTF-8', $day . " " . $month . " " . $year));
+                my $es = $settings->{entry_size};
+                if ($es > 8) {
+                        $es = 8;
+                }
+                $desc = Pango::FontDescription->from_string($settings->{entry_font} . ' ' . $es . 'px');
+                $layout->set_font_description($desc);
+
+                my ($wid2, $h2) = $layout->get_pixel_size();
+                $cr->save;
+                $cr->move_to($settings->{margin_left} + $i * $cell + $cell/2 - $wid2/2, $self->{bounding_box}[1] + $h);
+                Pango::Cairo::show_layout($cr, $layout);
+                $cr->restore();
+
+                if ($h + $h2 > $ymax) {
+                        $ymax = $h + $h2;
+                }
+        }
+        $self->{heading_bottom_y} = $ymax + $self->{bounding_box}[1];
+}
+
+sub draw_entries
+{
+        my ($self, $cr, $settings, $i) = @_;
+
+        my $cell = ($settings->{width} - $settings->{margin_left} - $settings->{margin_right})/7;
+
+        # Coordinates of box from line-to-line
+        my $l2l_box = [$i * $cell + $settings->{margin_left},
+                       $self->{heading_bottom_y},
+                       ($i+1) * $cell + $settings->{margin_left},
+                       $self->{bounding_box}[3]];
+
+        # Coordinates of drawing-space box
+        my $box = [$l2l_box->[0] + $settings->{border_size},
+                   $l2l_box->[1] + $settings->{border_size},
+                   $l2l_box->[2] - $settings->{border_size},
+                   $l2l_box->[3] - $settings->{border_size}];
+
+        $self->{l2l_box} = $l2l_box;
+        $self->{box} = $box;
+
+        # Do shading, if any
+        my $shade = $self->find_last_special('shade', $self->{entries}->[$i]);
+        if ($shade) {
+                $cr->save;
+                $cr->set_source_rgb($shade->{r} / 255,
+                                    $shade->{g} / 255,
+                                    $shade->{b} / 255);
+                $cr->rectangle($l2l_box->[0], $l2l_box->[1],
+                               $l2l_box->[2] - $l2l_box->[0],
+                               $l2l_box->[3] - $l2l_box->[1]);
+                $cr->fill();
+                $cr->restore;
+        }
+
+        # Get the "day number" size to leave room for moon and week specials
+        my $layout = Pango::Cairo::create_layout($cr);
+        $layout->set_text("31");
+        my $desc = Pango::FontDescription->from_string($settings->{daynum_font} . ' ' . $settings->{daynum_size} . 'px');
+
+        $layout->set_font_description($desc);
+        my ($wid, $h) = $layout->get_pixel_size();
+
+        my $so_far = $box->[1] + $h + $settings->{border_size};
+
+        my $box_height = $box->[3] - $box->[1];
+        my $done = 0;
+        foreach my $entry (@{$self->{entries}->[$i]}) {
+                # Moon and week should not adjust height
+                if ($entry->isa('Remind::PDF::Entry::moon') ||
+                    $entry->isa('Remind::PDF::Entry::week')) {
+                        $entry->render($self, $cr, $settings, $box->[1], $i, $i, $box_height);
                         next;
                 }
+
+                # An absolutely-positioned Pango markup should not adjust height
+                # either
+                if ($entry->isa('Remind::PDF::Entry::pango') &&
+                    defined($entry->{atx}) && defined($entry->{aty})) {
+                        $entry->render($self, $cr, $settings, $box->[1], $i, $i, $box_height);
+                        next;
+                }
+
+                # Shade is done already
+                if ($entry->isa('Remind::PDF::Entry::shade')) {
+                        next;
+                }
+                if ($done) {
+                        $so_far += $settings->{border_size};
+                }
                 $done = 1;
-                $e->render($cr, $settings);
+                my $h2 = $entry->render($self, $cr, $settings, $so_far, $i, $i, $box_height);
+                $so_far += $h2;
         }
 }
 
+sub col_box_coordinates
+{
+
+        my ($self, $so_far, $col, $height, $settings) = @_;
+        return (@{$self->{l2l_box}});
+}
+
+
+
+sub draw_lines
+{
+        my ($self, $cr, $settings) = @_;
+
+        # Top horizonal line
+        $cr->move_to($self->{bounding_box}[0], $self->{bounding_box}[1]);
+        $cr->line_to($self->{bounding_box}[2], $self->{bounding_box}[1]);
+        $cr->stroke();
+
+        # Horizontal line below headings
+        $cr->move_to($self->{bounding_box}[0], $self->{heading_bottom_y});
+        $cr->line_to($self->{bounding_box}[2], $self->{heading_bottom_y});
+        $cr->stroke();
+
+        # Bottom horizontal line
+        $cr->move_to($self->{bounding_box}[0], $self->{bounding_box}[3]);
+        $cr->line_to($self->{bounding_box}[2], $self->{bounding_box}[3]);
+        $cr->stroke();
+
+        # Vertical lines
+        my $w = ($settings->{width} - $settings->{margin_left} - $settings->{margin_right})/7;
+        for (my $i=0; $i<=7; $i++) {
+                my $x = $settings->{margin_left} + ($i * $w);
+                $cr->move_to($x, $self->{bounding_box}[1]);
+                $cr->line_to($x, $self->{bounding_box}[3]);
+                $cr->stroke();
+        }
+}
+
+sub create_from_hash
+{
+        my ($class, $hash, $specials_accepted) = @_;
+        bless $hash, $class;
+
+        my $filtered_entries = [];
+        my $date_to_index;
+        for (my $i=0; $i<7; $i++) {
+                $date_to_index->{$hash->{dates}[$i]->{date}} = $i;
+        }
+
+        for (my $i=0; $i<7; $i++) {
+                $filtered_entries->[$i] = [];
+        }
+        foreach my $e (@{$hash->{entries}}) {
+                if ($hash->accept_special($e, $specials_accepted)) {
+                        my $index = $date_to_index->{$e->{date}};
+                        push(@{$filtered_entries->[$index]}, Remind::PDF::Entry->new_from_hash($e));
+                }
+        }
+        $hash->{entries} = $filtered_entries;
+        return $hash;
+}
+
 1;
 
